"""Extension module supporting crs.py.

Calls methods from GDAL's OSR module.
"""

from __future__ import absolute_import

import logging
import os

from fiona cimport _cpl
from fiona._err cimport exc_wrap_pointer, exc_wrap_ogrerr
from fiona._err import CPLE_BaseError
from fiona.compat import DICT_TYPES
from fiona.errors import CRSError


logger = logging.getLogger(__name__)


cdef void osr_set_traditional_axis_mapping_strategy(OGRSpatialReferenceH hSrs):
    OSRSetAxisMappingStrategy(hSrs, OAMS_TRADITIONAL_GIS_ORDER)


cdef _osr_to_wkt(OGRSpatialReferenceH cogr_srs, crs, wkt_version):
    cdef char *wkt_c = NULL
    wkt = None
    cdef const char* options_wkt[2]
    options_wkt[0] = NULL
    options_wkt[1] = NULL

    if wkt_version:
        wkt_format = f"FORMAT={wkt_version}".encode("utf-8")
        options_wkt[0] = wkt_format

    gdal_error = None
    try:
        exc_wrap_ogrerr(
            OSRExportToWktEx(cogr_srs, &wkt_c, options_wkt)
        )
    except CPLE_BaseError as error:
        gdal_error = error

    if wkt_c != NULL:
        wkt_b = wkt_c
        wkt = wkt_b.decode('utf-8')

    if not wkt and wkt_version is None:
        # attempt to morph to ESRI before export
        options_wkt[0] = "FORMAT=WKT1_ESRI"
        try:
            exc_wrap_ogrerr(
                OSRExportToWktEx(cogr_srs, &wkt_c, options_wkt)
            )
        except CPLE_BaseError as error:
            gdal_error = error

    if wkt_c != NULL:
        wkt_b = wkt_c
        wkt = wkt_b.decode('utf-8')

    _cpl.CPLFree(wkt_c)

    if not wkt:
        error_message = (
            f"Unable to export CRS to WKT. "
            "Please choose a different WKT version (WKT2 is recommended): "
            "WKT1 | WKT1_GDAL | WKT1_ESRI | WKT2 | WKT2_2018 | WKT2_2015."
        )
        if gdal_error is not None:
            error_message = f"{error_message} GDAL ERROR: {gdal_error}"
        raise CRSError(error_message)
    return wkt


# Export a WKT string from input crs.
def crs_to_wkt(crs, wkt_version=None):
    """Convert a Fiona CRS object to WKT format"""
    cdef OGRSpatialReferenceH cogr_srs = NULL
    cdef char *proj_c = NULL

    try:
        cogr_srs = exc_wrap_pointer(OSRNewSpatialReference(NULL))
    except CPLE_BaseError as exc:
        raise CRSError(str(exc))

    # check for other CRS classes
    if hasattr(crs, "to_wkt") and callable(crs.to_wkt):
        crs = crs.to_wkt()

    # First, check for CRS strings like "EPSG:3857".
    if isinstance(crs, str):
        proj_b = crs.encode('utf-8')
        proj_c = proj_b
        OSRSetFromUserInput(cogr_srs, proj_c)

    elif isinstance(crs, DICT_TYPES):
        # EPSG is a special case.
        init = crs.get('init')
        if init:
            logger.debug("Init: %s", init)
            auth, val = init.split(':')
            if auth.upper() == 'EPSG':
                logger.debug("Setting EPSG: %s", val)
                OSRImportFromEPSG(cogr_srs, int(val))
        else:
            params = []
            crs['wktext'] = True
            for k, v in crs.items():
                if v is True or (k in ('no_defs', 'wktext') and v):
                    params.append("+%s" % k)
                else:
                    params.append("+%s=%s" % (k, v))
            proj = " ".join(params)
            logger.debug("PROJ.4 to be imported: %r", proj)
            proj_b = proj.encode('utf-8')
            proj_c = proj_b
            OSRImportFromProj4(cogr_srs, proj_c)
    else:
        raise CRSError("Invalid input to create CRS: {}".format(crs))

    osr_set_traditional_axis_mapping_strategy(cogr_srs)
    return _osr_to_wkt(cogr_srs, crs, wkt_version)



"""Fiona data model"""

from collections.abc import MutableMapping
from collections import OrderedDict
import itertools
from json import JSONEncoder
from warnings import warn
from collections.abc import MutableMapping

from fiona.errors import FionaDeprecationWarning


# Mapping of OGR integer geometry types to GeoJSON type names.
GEOMETRY_TYPES = {
    0: "Unknown",
    1: "Point",
    2: "LineString",
    3: "Polygon",
    4: "MultiPoint",
    5: "MultiLineString",
    6: "MultiPolygon",
    7: "GeometryCollection",
    # Unsupported types.
    # 8: 'CircularString',
    # 9: 'CompoundCurve',
    # 10: 'CurvePolygon',
    # 11: 'MultiCurve',
    # 12: 'MultiSurface',
    # 13: 'Curve',
    # 14: 'Surface',
    # 15: 'PolyhedralSurface',
    # 16: 'TIN',
    # 17: 'Triangle',
    100: "None",
    101: "LinearRing",
    0x80000001: "3D Point",
    0x80000002: "3D LineString",
    0x80000003: "3D Polygon",
    0x80000004: "3D MultiPoint",
    0x80000005: "3D MultiLineString",
    0x80000006: "3D MultiPolygon",
    0x80000007: "3D GeometryCollection",
}


# Mapping of OGR integer geometry types to GeoJSON type names.
GEOMETRY_TYPES = {
    0: "Unknown",
    1: "Point",
    2: "LineString",
    3: "Polygon",
    4: "MultiPoint",
    5: "MultiLineString",
    6: "MultiPolygon",
    7: "GeometryCollection",
    # Unsupported types.
    # 8: 'CircularString',
    # 9: 'CompoundCurve',
    # 10: 'CurvePolygon',
    # 11: 'MultiCurve',
    # 12: 'MultiSurface',
    # 13: 'Curve',
    # 14: 'Surface',
    # 15: 'PolyhedralSurface',
    # 16: 'TIN',
    # 17: 'Triangle',
    100: "None",
    101: "LinearRing",
    0x80000001: "3D Point",
    0x80000002: "3D LineString",
    0x80000003: "3D Polygon",
    0x80000004: "3D MultiPoint",
    0x80000005: "3D MultiLineString",
    0x80000006: "3D MultiPolygon",
    0x80000007: "3D GeometryCollection",
}


class Object(MutableMapping):
    """Base class for CRS, geometry, and feature objects

    In Fiona 2.0, the implementation of those objects will change.  They
    will no longer be dicts or derive from dict, and will lose some
    features like mutability and default JSON serialization.

    Object will be used for these objects in Fiona 1.9. This class warns
    about future deprecation of features.
    """

    _delegated_properties = []

    def __init__(self, **kwds):
        self._data = OrderedDict(**kwds)

    def _props(self):
        return {
            k: getattr(self._delegate, k)
            for k in self._delegated_properties
            if k is not None
        }

    def __getitem__(self, item):
        props = self._props()
        props.update(**self._data)
        return props[item]

    def __iter__(self):
        props = self._props()
        return itertools.chain(iter(props), iter(self._data))

    def __len__(self):
        props = self._props()
        return len(props) + len(self._data)

    def __setitem__(self, key, value):
        warn(
            "instances of this class -- CRS, geometry, and feature objects -- will become immutable in fiona version 2.0",
            FionaDeprecationWarning,
            stacklevel=2,
        )
        if key in self._delegated_properties:
            setattr(self._delegate, key, value)
        else:
            self._data[key] = value

    def __delitem__(self, key):
        warn(
            "instances of this class -- CRS, geometry, and feature objects -- will become immutable in fiona version 2.0",
            FionaDeprecationWarning,
            stacklevel=2,
        )
        if key in self._delegated_properties:
            setattr(self._delegate, key, None)
        else:
            del self._data[key]

    def __eq__(self, other):
        return dict(**self) == dict(**other)


class _Geometry(object):
    def __init__(self, coordinates=None, type=None):
        self.coordinates = coordinates
        self.type = type


class Geometry(Object):
    """A GeoJSON-like geometry

    Notes
    -----
    Delegates coordinates and type properties to an instance of
    _Geometry, which will become an extension class in Fiona 2.0.

    """

    _delegated_properties = ["coordinates", "type"]

    def __init__(self, coordinates=None, type=None, **data):
        self._delegate = _Geometry(coordinates=coordinates, type=type)
        super(Geometry, self).__init__(**data)

    @classmethod
    def from_dict(cls, mapping=None, **kwargs):
        data = dict(mapping or {}, **kwargs)
        return Geometry(
            coordinates=data.pop("coordinates", None),
            type=data.pop("type", None),
            **data
        )

    @property
    def coordinates(self):
        """The geometry's coordinates

        Returns
        -------
        Sequence

        """
        return self._delegate.coordinates

    @property
    def type(self):
        """The geometry's type

        Returns
        -------
        str

        """
        return self._delegate.type


class _Feature(object):
    def __init__(self, geometry=None, id=None, properties=None):
        self.geometry = geometry
        self.id = id
        self.properties = properties


class Feature(Object):
    """A GeoJSON-like feature

    Notes
    -----
    Delegates geometry and properties to an instance of _Feature, which
    will become an extension class in Fiona 2.0.

    """

    _delegated_properties = ["geometry", "id", "properties"]

    def __init__(self, geometry=None, id=None, properties=None, **data):
        self._delegate = _Feature(geometry=geometry, id=id, properties=properties)
        super(Feature, self).__init__(**data)

    @classmethod
    def from_dict(cls, mapping=None, **kwargs):
        data = dict(mapping or {}, **kwargs)
        geom_data = data.pop("geometry", None)

        if isinstance(geom_data, Geometry):
            geom = geom_data
        else:
            geom = (
                Geometry(
                    coordinates=geom_data.pop("coordinates", None),
                    type=geom_data.pop("type", None),
                    **geom_data
                )
                if geom_data is not None
                else None
            )

        props_data = data.pop("properties", None)

        if isinstance(props_data, Properties):
            props = props_data
        else:
            props = Properties(**props_data) if props_data is not None else None

        fid = data.pop("id", None)
        return Feature(geometry=geom, id=fid, properties=props, **data)

    def __eq__(self, other):
        return (
            self.geometry == other.geometry
            and self.id == other.id
            and self.properties == other.properties
        )

    @property
    def geometry(self):
        """The feature's geometry object

        Returns
        -------
        Geometry

        """
        return self._delegate.geometry

    @property
    def id(self):
        """The feature's id

        Returns
        ------
        obejct

        """
        return self._delegate.id

    @property
    def properties(self):
        """The feature's properties

        Returns
        -------
        Object

        """
        return self._delegate.properties

    @property
    def type(self):
        """The Feature's type

        Returns
        -------
        str

        """
        return "Feature"


class Properties(Object):
    """A GeoJSON-like feature's properties

    """

    def __init__(self, **kwds):
        super(Properties, self).__init__(**kwds)

    @classmethod
    def from_dict(cls, mapping=None, **kwargs):
        data = dict(mapping or {}, **kwargs)
        return Properties(**data)


class ObjectEncoder(JSONEncoder):
    """Encodes Geometry and Feature"""

    def default(self, o):
        if isinstance(o, (Geometry, Properties)):
            return dict(**o)
        elif isinstance(o, Feature):
            o_dict = dict(**o)
            o_dict["type"] = "Feature"
            if o.geometry is not None:
                o_dict["geometry"] = ObjectEncoder().default(o.geometry)
            if o.properties is not None:
                o_dict["properties"] = ObjectEncoder().default(o.properties)
            return o_dict
        else:
            return JSONEncoder().default(o)


def decode_object(o):
    """A json.loads object_hook

    Parameters
    ----------
    o : dict
        A decoded dict.

    Returns
    -------
    Feature, Geometry, or dict

    """
    if "type" in o:
        if o["type"] == "Feature":
            val = Feature.from_dict(**o)
        elif o["type"] in list(GEOMETRY_TYPES.values())[:8]:
            val = Geometry.from_dict(**o)
        else:
            val = o
    else:
        val = o

    return val


def to_dict(val):
    """Converts an object to a dict"""
    try:
        o = ObjectEncoder().default(val)
    except TypeError:
        pass
    else:
        return o

    return val



"""Tests of the transform submodule"""

import math
import pytest
from fiona import transform
from fiona.errors import FionaDeprecationWarning

from .conftest import requires_gdal_lt_3


TEST_GEOMS = [
    {"type": "Point", "coordinates": [0.0, 0.0, 1000.0]},
    {
        "type": "LineString",
        "coordinates": [[0.0, 0.0, 1000.0], [0.1, 0.1, -1000.0]],
    },
    {
        "type": "MultiPoint",
        "coordinates": [[0.0, 0.0, 1000.0], [0.1, 0.1, -1000.0]],
    },
    {
        "type": "Polygon",
        "coordinates": [
            [
                [0.0, 0.0, 1000.0],
                [0.1, 0.1, -1000.0],
                [0.1, -0.1, math.pi],
                [0.0, 0.0, 1000.0],
            ]
        ],
    },
    {
        "type": "MultiPolygon",
        "coordinates": [
            [
                [
                    [0.0, 0.0, 1000.0],
                    [0.1, 0.1, -1000.0],
                    [0.1, -0.1, math.pi],
                    [0.0, 0.0, 1000.0],
                ]
            ]
        ],
    },
]


@pytest.mark.parametrize("geom", TEST_GEOMS)
def test_transform_geom_with_z(geom):
    """Transforming a geom with Z succeeds"""
    with pytest.warns(FionaDeprecationWarning):
        transform.transform_geom("epsg:4326", "epsg:3857", geom, precision=3)


@pytest.mark.parametrize("geom", TEST_GEOMS)
def test_transform_geom_array_z(geom):
    """Transforming a geom array with Z succeeds"""
    g2 = transform.transform_geom(
        "epsg:4326",
        "epsg:3857",
        [geom for _ in range(5)],
        precision=3,
    )
    assert isinstance(g2, list)
    assert len(g2) == 5


@requires_gdal_lt_3
def test_transform_geom_null_dest():
    failed_geom = {
        'type': 'Polygon',
        'coordinates': ((
            (81.2180196471443, 6.197141424988303),
            (80.34835696810447, 5.968369859232141),
            (79.87246870312859, 6.763463446474915),
            (79.69516686393516, 8.200843410673372),
            (80.14780073437967, 9.824077663609557),
            (80.83881798698664, 9.268426825391174),
            (81.3043192890718, 8.564206244333675),
            (81.78795901889143, 7.523055324733178),
            (81.63732221876066, 6.481775214051936),
            (81.2180196471443, 6.197141424988303)
        ),)
    }
    with pytest.warns(UserWarning):
        transformed_geom = transform.transform_geom(
            src_crs="epsg:4326",
            dst_crs="epsg:32628",
            geom=failed_geom,
            antimeridian_cutting=True,
            precision=2,
        )
        assert transformed_geom is None


@pytest.mark.parametrize("crs", ["epsg:4326",
                                 "EPSG:4326",
                                 "WGS84",
                                 {'init': 'epsg:4326'},
                                 {'proj': 'longlat', 'datum': 'WGS84', 'no_defs': True},
                                 "OGC:CRS84"])
def test_axis_ordering(crs):
    """ Test if transform uses traditional_axis_mapping """

    expected = (-8427998.647958742, 4587905.27136252)
    t1 = transform.transform(crs, "epsg:3857", [-75.71], [38.06])
    assert (t1[0][0], t1[1][0]) == pytest.approx(expected)
    geom = {"type": "Point", "coordinates": [-75.71, 38.06]}
    g1 = transform.transform_geom(crs, "epsg:3857", geom, precision=3)
    assert g1["coordinates"] == pytest.approx(expected)

    rev_expected = (-75.71, 38.06)
    t2 = transform.transform("epsg:3857", crs, [-8427998.647958742], [4587905.27136252])
    assert (t2[0][0], t2[1][0]) == pytest.approx(rev_expected)
    geom = {"type": "Point", "coordinates": [-8427998.647958742, 4587905.27136252]}
    g2 = transform.transform_geom("epsg:3857", crs, geom, precision=3)
    assert g2["coordinates"] == pytest.approx(rev_expected)


def test_transform_issue971():
    """ See https://github.com/Toblerity/Fiona/issues/971 """
    source_crs = "epsg:25832"
    dest_src = "epsg:4326"
    geom = {'type': 'GeometryCollection', 'geometries': [{'type': 'LineString',
                                                          'coordinates': [(512381.8870945257, 5866313.311218272),
                                                                          (512371.23869999964, 5866322.282500001),
                                                                          (512364.6014999999, 5866328.260199999)]}]}
    geom_transformed = transform.transform_geom(source_crs, dest_src, geom, precision=3)
    assert geom_transformed['geometries'][0]['coordinates'][0] == pytest.approx((9.184, 52.946))



from distutils.command.sdist import sdist
from distutils import log
import itertools as it
import os
import shutil
import subprocess
import sys
from setuptools import setup
from setuptools.extension import Extension


# Use Cython if available.
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None


def check_output(cmd):
    return subprocess.check_output(cmd).decode('utf')

def copy_data_tree(datadir, destdir):
    try:
        shutil.rmtree(destdir)
    except OSError:
        pass
    shutil.copytree(datadir, destdir)


# Parse the version from the fiona module.
with open('fiona/__init__.py', 'r') as f:
    for line in f:
        if line.find("__version__") >= 0:
            version = line.split("=")[1].strip().strip('"').strip("'")
            break

# Fiona's auxiliary files are UTF-8 encoded
open_kwds = {'encoding': 'utf-8'}

with open('VERSION.txt', 'w', **open_kwds) as f:
    f.write(version)

with open('README.rst', **open_kwds) as f:
    readme = f.read()

with open('CREDITS.txt', **open_kwds) as f:
    credits = f.read()

with open('CHANGES.txt', **open_kwds) as f:
    changes = f.read()

# Building Fiona requires options that can be obtained from GDAL's gdal-config
# program or can be specified using setup arguments. The latter override the
# former.
#
# A GDAL API version is strictly required. Without this the setup script
# cannot know whether to use the GDAL version 1 or 2 source files. The GDAL
# API version can be specified in 2 ways.
#
# 1. By the gdal-config program, optionally pointed to by GDAL_CONFIG
# 2. By a GDAL_VERSION environment variable. This overrides number 1.


include_dirs = []
library_dirs = []
libraries = []
extra_link_args = []
gdal_output = [None for i in range(4)]
gdalversion = None
language = None
gdal_major_version = 0
gdal_minor_version = 0

if 'clean' not in sys.argv:
    try:
        gdal_config = os.environ.get('GDAL_CONFIG', 'gdal-config')
        for i, flag in enumerate(
                ["--cflags", "--libs", "--datadir", "--version"]):
            gdal_output[i] = check_output([gdal_config, flag]).strip()
        for item in gdal_output[0].split():
            if item.startswith("-I"):
                include_dirs.extend(item[2:].split(":"))
        for item in gdal_output[1].split():
            if item.startswith("-L"):
                library_dirs.extend(item[2:].split(":"))
            elif item.startswith("-l"):
                libraries.append(item[2:])
            else:
                # e.g. -framework GDAL
                extra_link_args.append(item)
        gdalversion = gdal_output[3]
        if gdalversion:
            log.info("GDAL API version obtained from gdal-config: %s",
                     gdalversion)

    except Exception as e:
        if os.name == "nt":
            log.info("Building on Windows requires extra options to setup.py "
                     "to locate needed GDAL files.\nMore information is "
                     "available in the README.")
        else:
            log.warn("Failed to get options via gdal-config: %s", str(e))

    # Get GDAL API version from environment variable.
    if 'GDAL_VERSION' in os.environ:
        gdalversion = os.environ['GDAL_VERSION']
        log.info("GDAL API version obtained from environment: %s", gdalversion)

    # Get GDAL API version from the command line if specified there.
    if '--gdalversion' in sys.argv:
        index = sys.argv.index('--gdalversion')
        sys.argv.pop(index)
        gdalversion = sys.argv.pop(index)
        log.info("GDAL API version obtained from command line option: %s",
                 gdalversion)

    if not gdalversion:
        log.fatal("A GDAL API version must be specified. Provide a path "
                  "to gdal-config using a GDAL_CONFIG environment variable "
                  "or use a GDAL_VERSION environment variable.")
        sys.exit(1)

    if os.environ.get('PACKAGE_DATA'):
        destdir = 'fiona/gdal_data'
        if gdal_output[2]:
            log.info("Copying gdal data from %s" % gdal_output[2])
            copy_data_tree(gdal_output[2], destdir)
        else:
            # check to see if GDAL_DATA is defined
            gdal_data = os.environ.get('GDAL_DATA', None)
            if gdal_data:
                log.info("Copying gdal data from %s" % gdal_data)
                copy_data_tree(gdal_data, destdir)

        # Conditionally copy PROJ.4 data.
        projdatadir = os.environ.get('PROJ_LIB', '/usr/local/share/proj')
        if os.path.exists(projdatadir):
            log.info("Copying proj data from %s" % projdatadir)
            copy_data_tree(projdatadir, 'fiona/proj_data')

    if "--cython-language" in sys.argv:
        index = sys.argv.index("--cython-language")
        sys.argv.pop(index)
        language = sys.argv.pop(index).lower()

    gdal_version_parts = gdalversion.split('.')
    gdal_major_version = int(gdal_version_parts[0])
    gdal_minor_version = int(gdal_version_parts[1])

    if (gdal_major_version, gdal_minor_version) < (3, 1):
        raise SystemExit(
            "ERROR: GDAL >= 3.1 is required for fiona. "
            "Please upgrade GDAL."
        )

    log.info("GDAL version major=%r minor=%r", gdal_major_version, gdal_minor_version)

compile_time_env = {
    "CTE_GDAL_MAJOR_VERSION": gdal_major_version,
    "CTE_GDAL_MINOR_VERSION": gdal_minor_version,
}

ext_options = dict(
    include_dirs=include_dirs,
    library_dirs=library_dirs,
    libraries=libraries,
    extra_link_args=extra_link_args,
    cython_compile_time_env=compile_time_env,
)

# Enable coverage for cython pyx files.
if os.environ.get('CYTHON_COVERAGE'):
    from Cython.Compiler.Options import get_directive_defaults
    directive_defaults = get_directive_defaults()
    directive_defaults['linetrace'] = True
    directive_defaults['binding'] = True

    ext_options.update(dict(
        define_macros=[("CYTHON_TRACE_NOGIL", "1")]))

# GDAL 2.3+ requires C++11

if language == "c++":
    ext_options["language"] = "c++"
    if sys.platform != "win32":
        ext_options["extra_compile_args"] = ["-std=c++11"]

ext_options_cpp = ext_options.copy()
if sys.platform != "win32":
    ext_options_cpp["extra_compile_args"] = ["-std=c++11"]


# Define the extension modules.
ext_modules = []

if "clean" not in sys.argv:
    # When building from a repo, Cython is required.
    log.info("MANIFEST.in found, presume a repo, cythonizing...")
    if not cythonize:
        raise SystemExit(
            "Cython.Build.cythonize not found. "
            "Cython is required to build fiona."
        )

    ext_modules = cythonize([
        Extension('fiona._geometry', ['fiona/_geometry.pyx'], **ext_options),
        Extension('fiona.schema', ['fiona/schema.pyx'], **ext_options),
        Extension('fiona._transform', ['fiona/_transform.pyx'], **ext_options_cpp),
        Extension('fiona._crs', ['fiona/_crs.pyx'], **ext_options),
        Extension('fiona._env', ['fiona/_env.pyx'], **ext_options),
        Extension('fiona._err', ['fiona/_err.pyx'], **ext_options),
        Extension('fiona.ogrext', ['fiona/ogrext.pyx'], **ext_options)
        ],
        compiler_directives={"language_level": "3"},
        compile_time_env=compile_time_env,
    )

requirements = [
    'attrs>=17',
    'certifi',
    'click>=4.0',
    'cligj>=0.5',
    'click-plugins>=1.0',
    'munch',
    "setuptools",
]

extras_require = {
    'calc': ['shapely'],
    's3': ['boto3>=1.2.4'],
    'test': ['pytest>=3', 'pytest-cov', 'boto3>=1.2.4']
}

extras_require['all'] = list(set(it.chain(*extras_require.values())))


setup_args = dict(
    metadata_version='1.2',
    name='Fiona',
    version=version,
    python_requires='>=3.7',
    requires_external='GDAL (>=3.1)',
    description="Fiona reads and writes spatial data files",
    license='BSD',
    keywords='gis vector feature data',
    author='Sean Gillies',
    author_email='sean.gillies@gmail.com',
    maintainer='Sean Gillies',
    maintainer_email='sean.gillies@gmail.com',
    url='https://github.com/Toblerity/Fiona',
    long_description=readme + "\n" + changes + "\n" + credits,
    package_dir={'': '.'},
    packages=['fiona', 'fiona.fio'],
    entry_points='''
        [console_scripts]
        fio=fiona.fio.main:main_group

        [fiona.fio_commands]
        bounds=fiona.fio.bounds:bounds
        calc=fiona.fio.calc:calc
        cat=fiona.fio.cat:cat
        collect=fiona.fio.collect:collect
        distrib=fiona.fio.distrib:distrib
        dump=fiona.fio.dump:dump
        env=fiona.fio.env:env
        filter=fiona.fio.filter:filter
        info=fiona.fio.info:info
        insp=fiona.fio.insp:insp
        load=fiona.fio.load:load
        ls=fiona.fio.ls:ls
        rm=fiona.fio.rm:rm
        ''',
    install_requires=requirements,
    extras_require=extras_require,
    ext_modules=ext_modules,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering :: GIS'])

if os.environ.get('PACKAGE_DATA'):
    setup_args['package_data'] = {'fiona': ['gdal_data/*', 'proj_data/*', '.libs/*', '.libs/licenses/*']}

setup(**setup_args)



# Coordinate system and transform API functions.
include "gdal.pxi"

cdef extern from "ogr_srs_api.h":
    ctypedef enum OSRAxisMappingStrategy:
        OAMS_TRADITIONAL_GIS_ORDER

    void OSRSetAxisMappingStrategy(OGRSpatialReferenceH hSRS, OSRAxisMappingStrategy)
    OGRErr OSRExportToWktEx(OGRSpatialReferenceH, char ** ppszResult,
                            const char* const* papszOptions)

cdef void osr_set_traditional_axis_mapping_strategy(OGRSpatialReferenceH hSrs)
